#ifndef GATE_H
#define GATE_H

#include "SpSimObj.H"
#include "SpDefineSimObj.H"
#include "SpDefineEvent.H"

#include "connector.H"

class S_Gate: public SpSimObj {
public:
	S_Gate() { }
	virtual void Init();
	void signal(char p, int v);
	void forward();
protected:
	RB_int size;
	RB_int *value;

	// Distinguish function of a particular gate.
	virtual int eval() = 0;
	virtual int initout() = 0;
	virtual int initin() = 0;
	virtual char idchar() = 0;
	virtual void dofwd(double, SpObjHandle) = 0;
};

class S_And_Gate: public S_Gate {
public:
	S_And_Gate() { }
private:
	virtual int eval();
	virtual int initout() { return 0; }
	virtual int initin() { return 0; }
	virtual char idchar() { return 'A'; }
	virtual void dofwd(double t, SpObjHandle h);
};

DEFINE_SIMOBJ(S_And_Gate, conobj->count('A'), SCATTER);
DEFINE_SIMOBJ_EVENT_2_ARG(a_signal, S_And_Gate, signal, char, int);
DEFINE_SIMOBJ_EVENT_0_ARG(a_fwd, S_And_Gate, forward);

class S_Or_Gate: public S_Gate {
public:
	S_Or_Gate() { }
private:
	virtual int eval();
	virtual int initout() { return 0; }
	virtual int initin() { return 0; }
	virtual char idchar() { return 'O'; }
	virtual void dofwd(double t, SpObjHandle h);
};

DEFINE_SIMOBJ(S_Or_Gate, conobj->count('O'), SCATTER);
DEFINE_SIMOBJ_EVENT_2_ARG(o_signal, S_Or_Gate, signal, char, int);
DEFINE_SIMOBJ_EVENT_0_ARG(o_fwd, S_Or_Gate, forward);

class S_Xor_Gate: public S_Gate {
public:
	S_Xor_Gate() { }
private:
	virtual int eval();
	virtual int initout() { return 0; }
	virtual int initin() { return 0; }
	virtual char idchar() { return 'X'; }
	virtual void dofwd(double t, SpObjHandle h);
};

DEFINE_SIMOBJ(S_Xor_Gate, conobj->count('X'), SCATTER);
DEFINE_SIMOBJ_EVENT_2_ARG(x_signal, S_Xor_Gate, signal, char, int);
DEFINE_SIMOBJ_EVENT_0_ARG(x_fwd, S_Xor_Gate, forward);

class S_Not_Gate: public S_Gate {
public:
	S_Not_Gate() { }
	void signal(int v) { S_Gate::signal('a', v); }
private:
	virtual int eval();
	virtual int initout() { return 0; }
	virtual int initin() { return 1; }
	virtual char idchar() { return 'N'; }
	virtual void dofwd(double t, SpObjHandle h);
};

DEFINE_SIMOBJ(S_Not_Gate, conobj->count('N'), SCATTER);
DEFINE_SIMOBJ_EVENT_1_ARG(n_signal, S_Not_Gate, signal, int);
DEFINE_SIMOBJ_EVENT_0_ARG(n_fwd, S_Not_Gate, forward);

#endif
